/*******************************************************************************
 * Copyright(c) 2023 Levetop Semiconductor Co.,Led. All rights reserved.
 * @file     tftlcd.c
 * @author   UartTFT Application Team
 * @version  V0.0.1
 * @date     2023-01-12
 * @brief    This file contains the interface initialization and basic
 *           control functions of the TFT.
 ********************************************************************************/

#include "tftlcd.h"
#include "ioctrl_drv.h"
#include "ccm_drv.h"
#include "dmac_drv.h"
#include "delay.h"
#include "transport.h"
#include "function.h"
#include "dma.h"
#if (LT_TFT_PORT == 0)

/********************************************************************************
 * Function Name	: LCD_SPI_Port_Init
 * Description  	: Initialize GPIO for SPI.
 * Input        	: None
 * Output       	: None
 * Return       	: None
 *********************************************************************************/
void LCD_SPI_Port_Init(void)
{
	SPI_InitTypeDef SPI_InitStruct;
	SPI_InitStruct.SPI_Mode = SPI_Mode_Master;
	SPI_InitStruct.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
	SPI_InitStruct.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_2;
	SPI_InitStruct.SPI_CPHA = SPI_CPHA_1Edge;
	SPI_InitStruct.SPI_CPOL = SPI_CPOL_Low;
	SPI_InitStruct.SPI_DataSize = SPI_DataSize_8b;
	SPI_InitStruct.SPI_FirstBit = SPI_FirstBit_MSB;
	SPI_Init(TFTSPI, &SPI_InitStruct);

	// DMA_Init(DMA1_BASE_ADDR);
	TFTspi_dma_channel[TFTSPI_CHNUM]->DMA_DADDR = (uint32_t)&TFTSPI->SPIDR;
	TFTspi_dma_channel[TFTSPI_CHNUM]->DMA_CTRL = (SIEC | DNOCHG | M2P_DMA | DBSIZE_1 | SBSIZE_1 | SWIDTH_B | DWIDTH_B); // frame_size = 1x8 bits
	// TFTspi_dma_channel[TFTSPI_CHNUM]->DMA_CTRL_HIGH = length;    	//length max = 4095, trans size  = length*frame_size
	TFTspi_dma_channel[TFTSPI_CHNUM]->DMA_CFG = (HS_SEL_SRC_SOFT);
	TFTspi_dma_channel[TFTSPI_CHNUM]->DMA_CFG_HIGH = TFTSPI_DST_TX;
	m_dma_control->DMA_CONFIG = 1;
	TFTSPI->SPIDMACR &= (~0x03); // disable SPI TX RX DMA

	IOCTRL->SPI_CONTROL_REG |= 0x03; // enhance SPI driving capability
	// IOCTRL->SPI_CONTROL_REG &=~(1<<2);	//set SPI voltage conversion rate to high speed

	UART_ConfigGpio(SCI2,UART_RX, GPIO_OUTPUT); //RST
	UART_WriteGpioData(SCI2,UART_RX, Bit_SET);
	
	UART_ConfigGpio(SCI1, UART_RX, GPIO_OUTPUT);//EPORT_ConfigGpio(EPORT_PIN0, GPIO_OUTPUT);	//RS
	UART_WriteGpioData(SCI1,UART_RX, Bit_SET);//EPORT_WriteGpioData(EPORT_PIN0, Bit_SET);
	
	EPORT_ConfigGpio(EPORT_PIN1, GPIO_OUTPUT);	//CS
	EPORT_WriteGpioData(EPORT_PIN1, Bit_SET);
}

/********************************************************************************
 * Function Name	: LCD_SPI_ReadWriteByte
 * Description  	: Send a byte data with spi.
 * Input        	: - Send: data to send
 * Output       	: None
 * Return       	: None
 *********************************************************************************/
uint8_t LCD_SPI_ReadWriteByte(uint8_t Send)
{
	uint8_t ucTemp = 0;

	while (((TFTSPI->SPISRHW) & (SPISR_TXFFULL_MASK)))
		;
	TFTSPI->SPIDR = Send;
	while (!(0x80 & TFTSPI->SPISR))
		;
	while (((TFTSPI->SPISRHW) & (SPISR_RXFEMP_MASK)))
		;
	ucTemp = TFTSPI->SPIDR;
	return ucTemp;
}

/********************************************************************************
* Function Name	: LCD_SPI_WriteData_DMA
* Description  	: Send the data in the buffer with spi dma.
* Input        	: - Tx_Addr: buffer for data
				  - length: bytes to send
* Output       	: None
* Return       	: None
*********************************************************************************/
void LCD_SPI_WriteData_DMA(uint8_t *Tx_Addr, uint32_t length)
{
	uint32_t i;
	uint16_t count0, count1;
	count0 = length / 0xC00;
	count1 = length % 0xC00;

	TFTSPI->SPIDMACR |= (SPISR_TXDMAE_MASK | SPISR_RXDMAE_MASK); // enable SPI TX RXDMA
	TFTspi_dma_control->DMA_CONFIG = 1;

	for (i = 0; i < count0; i++)
	{
		TFTspi_dma_channel[TFTSPI_CHNUM]->DMA_SADDR = (uint32_t)&Tx_Addr[i * 0xC00];
		TFTspi_dma_channel[TFTSPI_CHNUM]->DMA_CTRL_HIGH = 0xC00; // length max = 4095, trans size  = length*frame_size
		TFTspi_dma_control->DMA_CHEN |= CHANNEL_WRITE_ENABLE(TFTSPI_CHNUM) | CHANNEL_ENABLE(TFTSPI_CHNUM);

		while (TFTspi_dma_control->DMA_CHEN & CHANNEL_STAT(TFTSPI_CHNUM))
		{
			;
		}
		TFTspi_dma_control->DMA_CHEN &= ~(CHANNEL_WRITE_ENABLE(TFTSPI_CHNUM) | CHANNEL_ENABLE(TFTSPI_CHNUM));
		// TFTspi_dma_channel[TFTSPI_CHNUM]->DMA_CFG &= ~(1<<8);
	}
	if (count1 != 0)
	{
		TFTspi_dma_channel[TFTSPI_CHNUM]->DMA_SADDR = (uint32_t)&Tx_Addr[i * 0xC00];
		TFTspi_dma_channel[TFTSPI_CHNUM]->DMA_CTRL_HIGH = count1; // length max = 4095, trans size  = length*frame_size
		TFTspi_dma_control->DMA_CHEN |= CHANNEL_WRITE_ENABLE(TFTSPI_CHNUM) | CHANNEL_ENABLE(TFTSPI_CHNUM);

		while (TFTspi_dma_control->DMA_CHEN & CHANNEL_STAT(TFTSPI_CHNUM))
		{
			;
		}
		TFTspi_dma_control->DMA_CHEN &= ~(CHANNEL_WRITE_ENABLE(TFTSPI_CHNUM) | CHANNEL_ENABLE(TFTSPI_CHNUM));
		// TFTspi_dma_channel[TFTSPI_CHNUM]->DMA_CFG &= ~(1<<8);
	}

	TFTSPI->SPIDMACR &= (~(SPISR_TXDMAE_MASK | SPISR_RXDMAE_MASK));
}
#endif

/********************************************************************************
 * Function Name	: LCD_PPI_Port_Init
 * Description  	: Initialize GPIO for 8080 8bits/16bits .
 * Input        	: None
 * Output       	: None
 * Return       	: None
 *********************************************************************************/
void LCD_PPI_Port_Init(void)
{
#if (LT_TFT_PORT == 1)

	IO_Ctrl_SCI_Swap(TX3_GINT1,FALSE); 
	IO_Ctrl_SCI_Swap(RX3_GINT2,FALSE);
	UART_ConfigGpio(SCI2,UART_RX, GPIO_OUTPUT); //RST
	UART_WriteGpioData(SCI2,UART_RX, Bit_SET);
	UART_ConfigGpio(SCI2,UART_TX, GPIO_OUTPUT); //RD
	UART_WriteGpioData(SCI2,UART_TX, Bit_SET);
	EPORT_ConfigGpio(EPORT_PIN38, GPIO_OUTPUT);	//WR
	EPORT_WriteGpioData(EPORT_PIN38, Bit_SET);
	EPORT_ConfigGpio(EPORT_PIN0, GPIO_OUTPUT);	//RS
	EPORT_WriteGpioData(EPORT_PIN0, Bit_SET);
	EPORT_ConfigGpio(EPORT_PIN1, GPIO_OUTPUT);	//CS
	EPORT_WriteGpioData(EPORT_PIN1, Bit_SET);
	
	SPI_ConfigGpio(SPI3, SPI_SS, GPIO_OUTPUT);	//DB7
	SPI_ConfigGpio(SPI3, SPI_SCK, GPIO_OUTPUT);	//DB6	
	SPI_ConfigGpio(SPI3, SPI_MOSI, GPIO_OUTPUT);//DB5
	SPI_ConfigGpio(SPI3, SPI_MISO, GPIO_OUTPUT);//DB4	
	SPI_ConfigGpio(SPI1, SPI_SS, GPIO_OUTPUT);	//DB3
	SPI_ConfigGpio(SPI1, SPI_SCK, GPIO_OUTPUT);	//DB2
	SPI_ConfigGpio(SPI1, SPI_MOSI, GPIO_OUTPUT);//DB1
	SPI_ConfigGpio(SPI1, SPI_MISO, GPIO_OUTPUT);//DB0

#endif

#if (LT_TFT_PORT == 2)

	// PWM[1]/GINT[13]
	PWM_INPUT_OUTPUT_EN;
	PWM_ConfigGpio(PWM_PORT1, GPIO_OUTPUT, PWM_PULLUP_EN); // RST
	PWM_WriteGpioData(PWM_PORT1, Bit_SET);

	// PWM[0]/GINT[12]
	PWM_INPUT_OUTPUT_EN;
	PWM_ConfigGpio(PWM_PORT0, GPIO_OUTPUT, PWM_PULLUP_EN); // CS
	PWM_WriteGpioData(PWM_PORT0, Bit_SET);

	EPORT_ConfigGpio(EPORT_PIN7, GPIO_OUTPUT); // RS
	EPORT_WriteGpioData(EPORT_PIN7, Bit_SET);

	SPI_ConfigGpio(SPI3, SPI_MOSI, GPIO_OUTPUT); // RD
	SPI_WriteGpioData(SPI3, SPI_MOSI, Bit_SET);

	SPI_ConfigGpio(SPI3, SPI_SCK, GPIO_OUTPUT); // WR
	SPI_WriteGpioData(SPI3, SPI_SCK, Bit_SET);

	EPORT_ConfigGpio(EPORT_PIN24, GPIO_OUTPUT); // DB0
	EPORT_ConfigGpio(EPORT_PIN25, GPIO_OUTPUT); // DB1
	EPORT_ConfigGpio(EPORT_PIN26, GPIO_OUTPUT); // DB2
	EPORT_ConfigGpio(EPORT_PIN27, GPIO_OUTPUT); // DB3
	EPORT_ConfigGpio(EPORT_PIN28, GPIO_OUTPUT); // DB4
	EPORT_ConfigGpio(EPORT_PIN29, GPIO_OUTPUT); // DB5
	EPORT_ConfigGpio(EPORT_PIN30, GPIO_OUTPUT); // DB6
	EPORT_ConfigGpio(EPORT_PIN31, GPIO_OUTPUT); // DB7

	EPORT3->EPDDR = 0xFF; // DB0-DB7
	EPORT3->EPPUER = 0xFF;
	EPORT3->EPDR = 0x00;

#endif
}

// control RS pin of TFT
void LCD_RS(uint8_t bit)
{
	if (bit)
		LCD_RS_DATA;
	else
		LCD_RS_CMD;
}

// control CS pin of TFT
void LCD_CS(uint8_t bit)
{
	if (bit)
		LCD_CS_SET;
	else
		LCD_CS_RESET;
}

/********************************************************************************
 * Function Name	: TFT_DataWrite
 * Description  	: Send command to TFT.
 * Input        	: - cmd: command to be sent
 * Output       	: None
 * Return       	: None
 *********************************************************************************/
void TFT_CommandWrite(uint8_t cmd)
{
#if (LT_TFT_PORT == 0)
	LCD_RS_CMD;
	LCD_CS_RESET;
	LCD_SPI_ReadWriteByte(cmd);
	LCD_CS_SET;
#endif

#if (LT_TFT_PORT == 1 || LT_TFT_PORT == 2)
	LCD_RD_SET;
	LCD_RS_CMD;
	LCD_CS_RESET;

	LCD_DATA_WRITE_L(cmd);
	LCD_WR_RESET;
	LCD_WR_SET;
	LCD_CS_SET;
#endif
}

/********************************************************************************
 * Function Name	: TFT_DataWrite
 * Description  	: Send data to TFT.
 * Input        	: - data: data to be sent
 * Output       	: None
 * Return       	: None
 *********************************************************************************/
void TFT_DataWrite(uint8_t data)
{
#if (LT_TFT_PORT == 0)
	LCD_RS_DATA;
	LCD_CS_RESET;
	LCD_SPI_ReadWriteByte(data);
	LCD_CS_SET;
#endif

#if (LT_TFT_PORT == 1 || LT_TFT_PORT == 2)
	LCD_RD_SET;
	LCD_RS_DATA;
	LCD_CS_RESET;

	LCD_DATA_WRITE_L(data);
	LCD_WR_RESET;
	LCD_WR_SET;
	LCD_CS_SET;
#endif
}

/********************************************************************************
* Function Name	: TFT_DataReadReg
* Description  	: Read RGB data from TFT ram.
* Input        	: - buf: buffer for RGB data
				  - numpixel: Number of pixels to read
* Output       	: None
* Return       	: None
*********************************************************************************/
void TFT_DataRead(uint16_t *buf, uint32_t numpixel)
{
#if (LT_TFT_PORT == 1)

	uint8_t dey = 1;
	uint16_t r, g, b;

	TFT_CommandWrite(0x2e);
	LCD_CS_RESET;
	LCD_RS_DATA;
	LCD_PIN_INPUT;
	LCD_DATA_WRITE_L(0xFF);

	LCD_RD_RESET; // dummy
	DelayUS(dey);
	LCD_RD_SET;

	for (uint32_t i = 0; i < numpixel; i++)
	{
		LCD_RD_RESET;
		DelayUS(dey);
		LCD_RD_SET;
		r = LCD_DATA_READ_L & 0xF8;

		LCD_RD_RESET;
		DelayUS(dey);
		LCD_RD_SET;
		g = LCD_DATA_READ_L & 0xFC;

		LCD_RD_RESET;
		DelayUS(dey);
		LCD_RD_SET;
		b = LCD_DATA_READ_L & 0xF8;

		buf[i] = ((r << 8) | (g << 3) | (b >> 3));
	}
	LCD_CS_SET;
	LCD_PIN_OUTPUT;
#endif
}

/********************************************************************************
* Function Name	: TFT_DataReadReg
* Description  	: Read reg data from TFT.
* Input        	: - reg: TFT reg
				  - NumtoRead: bytes to read
* Output       	: None
* Return       	: None
*********************************************************************************/
uint32_t TFT_DataReadReg(uint8_t reg, uint8_t NumtoRead)
{
#if (LT_TFT_PORT == 1)
	uint32_t Val = 0;
	uint8_t dey = 1, i;

	TFT_CommandWrite(reg);
	LCD_CS_RESET;
	LCD_RS_DATA;
	LCD_PIN_INPUT;
	LCD_DATA_WRITE_L(0xFF);

	LCD_RD_RESET; // dummy
	DelayMS(dey);
	LCD_RD_SET;

	for (i = 0; i < NumtoRead; i++)
	{
		LCD_RD_RESET;
		DelayMS(dey);
		LCD_RD_SET;
		Val = LCD_DATA_READ_L;
		Val <<= 8;
	}
	LCD_CS_SET;
	LCD_PIN_OUTPUT;
	return Val;
#endif
}

/********************************************************************************
 * Function Name	: LCD_TFT_Init
 * Description  	: Initialize screen.
 * Input        	: None
 * Output       	: None
 * Return       	: None
 *********************************************************************************/
void LCD_TFT_Init(void)
{
#if (LT_TFT_PORT == 0)
	LCD_SPI_Port_Init();
#elif (LT_TFT_PORT == 1 || LT_TFT_PORT == 2)
	LCD_PPI_Port_Init();
#endif

	LCD_RST_SET;				//RST		
	DelayMS(10);
	LCD_RST_RESET;
	DelayMS(10);
	LCD_RST_SET;
	DelayMS(120); //Delay 120ms

#if 0
TFT_CommandWrite(0xFE);
TFT_CommandWrite(0xEF); 
TFT_CommandWrite(0xEB);
TFT_DataWrite(0x14); 

TFT_CommandWrite(0x84);
TFT_DataWrite(0x40); 

TFT_CommandWrite(0x85);
TFT_DataWrite(0xF1); 

TFT_CommandWrite(0x86);
TFT_DataWrite(0x98); 

TFT_CommandWrite(0x87);
TFT_DataWrite(0x28);

TFT_CommandWrite(0x88);
TFT_DataWrite(0x0A);

TFT_CommandWrite(0x89);
TFT_DataWrite(0x21); 

TFT_CommandWrite(0x8A);
TFT_DataWrite(0x00); 

TFT_CommandWrite(0x8B);
TFT_DataWrite(0x80); 

TFT_CommandWrite(0x8C);
TFT_DataWrite(0x01); 

TFT_CommandWrite(0x8D);
TFT_DataWrite(0x00); 

TFT_CommandWrite(0x8E);
TFT_DataWrite(0xDF); 

TFT_CommandWrite(0x8F);
TFT_DataWrite(0x52); 

TFT_CommandWrite(0xB6);
TFT_DataWrite(0x20); 

TFT_CommandWrite(0x36); //MY MX MV ML BGR MH 0 0 00
TFT_DataWrite(0x48);

TFT_CommandWrite(0x3A);
TFT_DataWrite(0x05); 

TFT_CommandWrite(0x90);
TFT_DataWrite(0x08);
TFT_DataWrite(0x08);
TFT_DataWrite(0x08);
TFT_DataWrite(0x08); 

TFT_CommandWrite(0xBD);
TFT_DataWrite(0x06);

TFT_CommandWrite(0xA6);
TFT_DataWrite(0x74);

TFT_CommandWrite(0xBF);
TFT_DataWrite(0x1C);

TFT_CommandWrite(0xA7);
TFT_DataWrite(0x45);
  
TFT_CommandWrite(0xA9);
TFT_DataWrite(0xBB);

TFT_CommandWrite(0xB8);
TFT_DataWrite(0x63);

TFT_CommandWrite(0xE8);
TFT_DataWrite(0x34);

TFT_CommandWrite(0xFF);
TFT_DataWrite(0x60);
TFT_DataWrite(0x01);
TFT_DataWrite(0x04);

TFT_CommandWrite(0x74);
TFT_DataWrite(0x10);
TFT_DataWrite(0x75);
TFT_DataWrite(0x80);
TFT_DataWrite(0x00);
TFT_DataWrite(0x00);
TFT_DataWrite(0x4E);
TFT_DataWrite(0x00);

TFT_CommandWrite(0xC3);
TFT_DataWrite(0x14); 
TFT_CommandWrite(0xC4);
TFT_DataWrite(0x14); 
TFT_CommandWrite(0xC9);
TFT_DataWrite(0x25);

TFT_CommandWrite(0xBE);
TFT_DataWrite(0x11);

TFT_CommandWrite(0xBC);
TFT_DataWrite(0x00);

TFT_CommandWrite(0xE1);
TFT_DataWrite(0x10);
TFT_DataWrite(0x0e);

TFT_CommandWrite(0xDF);
TFT_DataWrite(0x21);
TFT_DataWrite(0x0c);
TFT_DataWrite(0x02);

TFT_CommandWrite(0xED);
TFT_DataWrite(0x1B); 
TFT_DataWrite(0x0B); 

TFT_CommandWrite(0xAC);
TFT_DataWrite(0x47); 


TFT_CommandWrite(0xAE);
TFT_DataWrite(0x77);
TFT_CommandWrite(0xCB);
TFT_DataWrite(0x02);
TFT_CommandWrite(0xCD);
TFT_DataWrite(0x63);

TFT_CommandWrite(0x70);
TFT_DataWrite(0x07);
TFT_DataWrite(0x09);
TFT_DataWrite(0x04);
TFT_DataWrite(0x0e);
TFT_DataWrite(0x0F); 
TFT_DataWrite(0x09);
TFT_DataWrite(0x07);
TFT_DataWrite(0x08);
TFT_DataWrite(0x03);

TFT_CommandWrite(0xf0);
TFT_DataWrite(0x45);
TFT_DataWrite(0x09);
TFT_DataWrite(0x08);
TFT_DataWrite(0x08);
TFT_DataWrite(0x26);
TFT_DataWrite(0x2a);

TFT_CommandWrite(0xf1);
TFT_DataWrite(0x43);
TFT_DataWrite(0x70);
TFT_DataWrite(0x72);
TFT_DataWrite(0x36);
TFT_DataWrite(0x37);
TFT_DataWrite(0x6f);  

TFT_CommandWrite(0xf2);
TFT_DataWrite(0x45);
TFT_DataWrite(0x09);
TFT_DataWrite(0x08);
TFT_DataWrite(0x08);
TFT_DataWrite(0x26);
TFT_DataWrite(0x2a);

TFT_CommandWrite(0xf3);
TFT_DataWrite(0x43);
TFT_DataWrite(0x70);
TFT_DataWrite(0x72);
TFT_DataWrite(0x36);
TFT_DataWrite(0x37);
TFT_DataWrite(0x6f);

TFT_CommandWrite(0x62); 
TFT_DataWrite(0x18); 
TFT_DataWrite(0x0D);
TFT_DataWrite(0x71);
TFT_DataWrite(0xED);
TFT_DataWrite(0x70);
TFT_DataWrite(0x70);
TFT_DataWrite(0x18); 
TFT_DataWrite(0x0F);
TFT_DataWrite(0x71);
TFT_DataWrite(0xEF);
TFT_DataWrite(0x70);
TFT_DataWrite(0x70);

TFT_CommandWrite(0x63);
TFT_DataWrite(0x18);
TFT_DataWrite(0x11);
TFT_DataWrite(0x71);
TFT_DataWrite(0xF1);
TFT_DataWrite(0x70);
TFT_DataWrite(0x70);
TFT_DataWrite(0x18);
TFT_DataWrite(0x13);
TFT_DataWrite(0x71);
TFT_DataWrite(0xF3);
TFT_DataWrite(0x70);
TFT_DataWrite(0x70);

TFT_CommandWrite(0x64);
TFT_DataWrite(0x28);
TFT_DataWrite(0x29);
TFT_DataWrite(0xF1);
TFT_DataWrite(0x01);
TFT_DataWrite(0xF1); 
TFT_DataWrite(0x00);
TFT_DataWrite(0x07);

TFT_CommandWrite(0x66);
TFT_DataWrite(0x3C);
TFT_DataWrite(0x00);
TFT_DataWrite(0xCD);
TFT_DataWrite(0x67);
TFT_DataWrite(0x45);
TFT_DataWrite(0x45);
TFT_DataWrite(0x10);
TFT_DataWrite(0x00);
TFT_DataWrite(0x00);
TFT_DataWrite(0x00);

TFT_CommandWrite(0x67);
TFT_DataWrite(0x00);
TFT_DataWrite(0x3C);
TFT_DataWrite(0x00);
TFT_DataWrite(0x00);
TFT_DataWrite(0x00);
TFT_DataWrite(0x01);
TFT_DataWrite(0x54);
TFT_DataWrite(0x10);
TFT_DataWrite(0x32);
TFT_DataWrite(0x98);

TFT_CommandWrite(0x35);
TFT_CommandWrite(0x21);
DelayMS(120);
TFT_CommandWrite(0x11);
DelayMS(120);                  //ʱ120ms
TFT_CommandWrite(0x29);
DelayMS(120);
#else
	
	TFT_CommandWrite(0xFE);			 
	TFT_CommandWrite(0xEF); 

	TFT_CommandWrite(0xEB);	
	TFT_DataWrite(0x14); 

	TFT_CommandWrite(0x84);			
	TFT_DataWrite(0x60);//40 

	TFT_CommandWrite(0x85);
	TFT_DataWrite(0xFF); 

	TFT_CommandWrite(0x86);            
	TFT_DataWrite(0xFF); 

	TFT_CommandWrite(0x87);            
	TFT_DataWrite(0xFF);

	TFT_CommandWrite(0x8E);        
	TFT_DataWrite(0xFF); 

	TFT_CommandWrite(0x8F);     
	TFT_DataWrite(0xFF); 


	TFT_CommandWrite(0x88);		
	TFT_DataWrite(0x0A);

	TFT_CommandWrite(0x89);			
	TFT_DataWrite(0x21); 

	TFT_CommandWrite(0x8A);			
	TFT_DataWrite(0x00); 

	TFT_CommandWrite(0x8B);			
	TFT_DataWrite(0x80); 

	TFT_CommandWrite(0x8C);			
	TFT_DataWrite(0x01); 

	TFT_CommandWrite(0x8D);			
	TFT_DataWrite(0x03); 

	//TFT_CommandWrite(0xB5);//add
	//TFT_DataWrite(0x08); 
	// TFT_DataWrite(0x09); 
	// TFT_DataWrite(0x14);
	// TFT_DataWrite(0x08);
	//??
	TFT_CommandWrite(0xB6);
	TFT_DataWrite(0x00); 
	TFT_DataWrite(0x00); 
	

	TFT_CommandWrite(0x36);			
	TFT_DataWrite(0x48);

	TFT_CommandWrite(0x3A);			
	TFT_DataWrite(0x05); 


	TFT_CommandWrite(0x90);			
	TFT_DataWrite(0x08);
	TFT_DataWrite(0x08);
	TFT_DataWrite(0x08);
	TFT_DataWrite(0x08);
	
	TFT_CommandWrite(0xBA);			
	TFT_DataWrite(0x01);
	
	TFT_CommandWrite(0xBD);			
	TFT_DataWrite(0x06);

	TFT_CommandWrite(0xBC);			
	TFT_DataWrite(0x00);	

	TFT_CommandWrite(0xFF);			
	TFT_DataWrite(0x60);
	TFT_DataWrite(0x01);
	TFT_DataWrite(0x04);

	TFT_CommandWrite(0xC3);			
	TFT_DataWrite(0x10);// 15// 1d
	TFT_CommandWrite(0xC4);			
	TFT_DataWrite(0x10);// 15// 1d

	TFT_CommandWrite(0xC9);			
	TFT_DataWrite(0x25);

	TFT_CommandWrite(0xBE);			
	TFT_DataWrite(0x11); 

	TFT_CommandWrite(0xE1);
	TFT_DataWrite(0x10);
	TFT_DataWrite(0x0E);

	TFT_CommandWrite(0xDF);			
	TFT_DataWrite(0x21);
	TFT_DataWrite(0x0c);
	TFT_DataWrite(0x02);

	TFT_CommandWrite(0xF0);   
	TFT_DataWrite(0x45);
	TFT_DataWrite(0x09);
	TFT_DataWrite(0x08);
	TFT_DataWrite(0x08);
	TFT_DataWrite(0x26);
	TFT_DataWrite(0x2A);

	TFT_CommandWrite(0xF1);    
	TFT_DataWrite(0x43);
	TFT_DataWrite(0x70);
	TFT_DataWrite(0x72);
	TFT_DataWrite(0x36);
	TFT_DataWrite(0x37);  
	TFT_DataWrite(0x6F);


	TFT_CommandWrite(0xF2);   
	TFT_DataWrite(0x45);
	TFT_DataWrite(0x09);
	TFT_DataWrite(0x08);
	TFT_DataWrite(0x08);
	TFT_DataWrite(0x26);
	TFT_DataWrite(0x2A);

	TFT_CommandWrite(0xF3);   
	TFT_DataWrite(0x43);
	TFT_DataWrite(0x70);
	TFT_DataWrite(0x72);
	TFT_DataWrite(0x36);
	TFT_DataWrite(0x37); 
	TFT_DataWrite(0x6F);

	TFT_CommandWrite(0xED);	
	TFT_DataWrite(0x1B); 
	TFT_DataWrite(0x0B); 

	TFT_CommandWrite(0xAE);			
	TFT_DataWrite(0x77);

	TFT_CommandWrite(0xCD);			
	TFT_DataWrite(0x63);		


	TFT_CommandWrite(0x70);			
	TFT_DataWrite(0x07);
	TFT_DataWrite(0x07);
	TFT_DataWrite(0x04);
	TFT_DataWrite(0x0E); 
	TFT_DataWrite(0x10); 
	TFT_DataWrite(0x09);
	TFT_DataWrite(0x07);
	TFT_DataWrite(0x08);
	TFT_DataWrite(0x03);

	TFT_CommandWrite(0xE8);			
	TFT_DataWrite(0x34);
	/////////////////////////////////////////////
	/*TFT_CommandWrite(0x60);		
	TFT_DataWrite(0x38);
	TFT_DataWrite(0x0B);
	TFT_DataWrite(0x6D);
	TFT_DataWrite(0x6D);

	TFT_DataWrite(0x39);
	TFT_DataWrite(0xF0);
	TFT_DataWrite(0x6D);
	TFT_DataWrite(0x6D);


	TFT_CommandWrite(0x61);
	TFT_DataWrite(0x38);
	TFT_DataWrite(0xF4);
	TFT_DataWrite(0x6D);
	TFT_DataWrite(0x6D);

	TFT_DataWrite(0x38);
	//TFT_DataWrite(0xF7);
	TFT_DataWrite(0xF7);
	TFT_DataWrite(0x6D);
	TFT_DataWrite(0x6D);*/
	/////////////////////////////////////
	TFT_CommandWrite(0x62);
	TFT_DataWrite(0x18);
	TFT_DataWrite(0x0D);
	TFT_DataWrite(0x71);
	TFT_DataWrite(0xED);
	TFT_DataWrite(0x70);
	TFT_DataWrite(0x70);
	TFT_DataWrite(0x18);
	TFT_DataWrite(0x0F);
	TFT_DataWrite(0x71);
	TFT_DataWrite(0xEF);
	TFT_DataWrite(0x70); 
	TFT_DataWrite(0x70);

	TFT_CommandWrite(0x63);			
	TFT_DataWrite(0x18);
	TFT_DataWrite(0x11);
	TFT_DataWrite(0x71);
	TFT_DataWrite(0xF1);
	TFT_DataWrite(0x70);
	TFT_DataWrite(0x70);
	TFT_DataWrite(0x18);
	TFT_DataWrite(0x13);
	TFT_DataWrite(0x71);
	TFT_DataWrite(0xF3);
	TFT_DataWrite(0x70); 
	TFT_DataWrite(0x70);
	///////////////////////////////////////////////////////
	TFT_CommandWrite(0x64);			
	TFT_DataWrite(0x3b);
	TFT_DataWrite(0x29);
	TFT_DataWrite(0xF1);
	TFT_DataWrite(0x01);
	TFT_DataWrite(0xF1);
	TFT_DataWrite(0x00);
	TFT_DataWrite(0x0a);

	//??
	TFT_CommandWrite(0x66);			
	TFT_DataWrite(0x3C);
	TFT_DataWrite(0x00);
	TFT_DataWrite(0xCD);
	TFT_DataWrite(0x67);
	TFT_DataWrite(0x45);
	TFT_DataWrite(0x45);
	TFT_DataWrite(0x10);
	TFT_DataWrite(0x00);
	TFT_DataWrite(0x00);
	TFT_DataWrite(0x00);

	TFT_CommandWrite(0x67);			
	TFT_DataWrite(0x00);
	TFT_DataWrite(0x3C);
	TFT_DataWrite(0x00);
	TFT_DataWrite(0x00);
	TFT_DataWrite(0x00);
	TFT_DataWrite(0x01);
	TFT_DataWrite(0x54);
	TFT_DataWrite(0x10);
	TFT_DataWrite(0x32);
	TFT_DataWrite(0x98);


					
	TFT_CommandWrite(0x74);			
	TFT_DataWrite(0x10);	
	TFT_DataWrite(0x68);	// 85// BC
	TFT_DataWrite(0x80);
	TFT_DataWrite(0x00); 
	TFT_DataWrite(0x00); 
	TFT_DataWrite(0x4E);
	TFT_DataWrite(0x00);					

//   TFT_CommandWrite(0x8d);			
//	TFT_DataWrite(0x03);
	
    TFT_CommandWrite(0x98);			
	TFT_DataWrite(0x3e);
	TFT_DataWrite(0x07);

	TFT_CommandWrite(0x99);			
	TFT_DataWrite(0x3e);
	TFT_DataWrite(0x07);


	TFT_CommandWrite(0x35);	
	TFT_DataWrite(0x00); 
	TFT_CommandWrite(0x44);			
	TFT_DataWrite(0x00);
	TFT_DataWrite(0xF7);
	TFT_CommandWrite(0x21);
	DelayMS(120);
	//--------end gamma setting--------------//

	TFT_CommandWrite(0x11);
	DelayMS(120);
	TFT_CommandWrite(0x29);
	DelayMS(20);
	TFT_CommandWrite(0x2C);
	DelayMS(10);
#endif
}

//------------------------screen window----------------------------------------------------------
// X-axis coordinate offset: XSoff = start offsetXSoff = end offset
// Y-axis coordinate offset: YSoff = start offsetYEoff = end offset
#define XSoff 0
#define XEoff 0
#define YSoff 0
#define YEoff 0
/********************************************************************************
* Function Name	: LT268_TFT_SetWindows
* Description  	: Set the area of the window on the screen.
* Input        	: - XStar,yStar: the starting coordinate of the window on the screen
				  - Width,High: width and high of the window on the screen
* Output       	: None
* Return       	: None
*********************************************************************************/
void LT268_TFT_SetWindows(uint16_t XStar, uint16_t yStar, uint16_t Width, uint16_t High)
{
	uint16_t XS, XE, YS, YE;
	XS = XStar + XSoff;
	XE = XStar + XEoff + Width - 1;
	YS = yStar + YSoff;
	YE = yStar + YEoff + High - 1;

	/*X-axis*/
	TFT_CommandWrite(0x2A);
	TFT_DataWrite(XS >> 8);
	TFT_DataWrite(XS);
	TFT_DataWrite(XE >> 8);
	TFT_DataWrite(XE);

	/*Y-axis*/
	TFT_CommandWrite(0x2B);
	TFT_DataWrite(YS >> 8);
	TFT_DataWrite(YS);
	TFT_DataWrite(YE >> 8);
	TFT_DataWrite(YE);

	TFT_CommandWrite(0x2c);
}

//---------------------TFT Sleep Mode----------------------
void ST7789VPanelSleepInMode(void)
{
	TFT_CommandWrite(0x28);
	TFT_CommandWrite(0x10);
	DelayMS(120);
}

//---------------------TFT Sleep Out----------------------
void ST7789VPanelSleepOutMode(void)
{
	TFT_CommandWrite(0x11);
	DelayMS(120);
	TFT_CommandWrite(0x29);
}
//---------------------TFT display on/off ----------------------
void LT268_Display_ON(void)
{
	TFT_CommandWrite(0x29);
}
void LT268_Display_OFF(void)
{
	TFT_CommandWrite(0x28);
}

//--------------------LCD_PIN_TEST--------------------------------
void LCD_PIN_TEST(void)
{
	while (1)
	{

#if (LT_TFT_PORT == 1)

		LCD_RST_SET;
		LCD_CS_SET;
		LCD_WR_SET;
		LCD_RD_SET;
		LCD_RS_DATA;
		LCD_DATA_WRITE_L(0xff);
		DelayMS(1000);

		LCD_RST_RESET;
		LCD_CS_RESET;
		LCD_WR_RESET;
		LCD_RD_RESET;
		LCD_RS_CMD;
		LCD_DATA_WRITE_L(0x00);
		DelayMS(1000);
#endif
	}
}
//------------------------------------------------------------------------------------------------------------